package com.gic.haoban.manage.web.controller.target;

import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.commons.web.qo.PageQo;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.api.dto.StoreDTO;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.manage.web.qo.target.QueryDataStatisticsCommonQO;
import com.gic.haoban.manage.web.utils.StoreAuthUtils;
import com.gic.haoban.manage.web.utils.target.DataTargetHttpUtils;
import com.gic.haoban.manage.web.utils.target.TargetAnalysisUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 好办小程序-目标跟进
 * @Author guojx
 * @Date 2023/1/17 9:51
 */
@Controller
public class TargetController {
    @Autowired
    private StoreAuthUtils storeAuthUtils;
    @Autowired
    private StoreService storeService;

    /**
     * 目标跟进达成分析数据概览
     * @param qo
     * @return
     */
    @RequestMapping(value = "get-target-attainment-analysis-overview")
    @ResponseBody
    public RestResponse getTargetAttainmentAnalysisOverview(QueryDataStatisticsCommonQO qo) {
        JSONObject jsonObject = getCommon(qo, false);
        Map<String, Object> res = DataTargetHttpUtils.http(DataTargetHttpUtils.getParamNoPage(jsonObject).toJSONString(),
                "data_mbr_target_store_total");
        JSONObject result1 = DataTargetHttpUtils.getData(res);
        //合并
        JSONObject jsonObject2 = getCommon(qo, true);
        Map<String, Object> res2 = DataTargetHttpUtils.http(DataTargetHttpUtils.getParamNoPage(jsonObject2).toJSONString(),
                qo.getTimeType() == 1 ? "data_mbr_target_attainment_analysis_year_overview" : "data_mbr_target_attainment_analysis_month_overview");
        JSONObject result2 = DataTargetHttpUtils.getData(res2);
        result1.putAll(result2);

        List<String> keyList = TargetAnalysisUtils.map.get(1);
        keyList.addAll(TargetAnalysisUtils.map.get(0));
        keyList.addAll(TargetAnalysisUtils.map.get(2));
        keyList.addAll(TargetAnalysisUtils.map.get(3));
        keyList.addAll(TargetAnalysisUtils.map.get(4));
        keyList.addAll(TargetAnalysisUtils.map.get(5));
        JSONObject result = new JSONObject();
        for (String key : keyList) {
            result.put(key, result1.get(key));
        }
        return RestResponse.successResult(result);
    }

    /**
     * 目标跟进核心目标达成概况
     * @return
     */
    @RequestMapping(value = "get-target-reach-overview")
    @ResponseBody
    public RestResponse getTargetReachOverview(QueryDataStatisticsCommonQO qo) {
        JSONObject jsonObject = getCommon(qo, false);
        Map<String, Object> res = DataTargetHttpUtils.http(DataTargetHttpUtils.getParamNoPage(jsonObject).toJSONString(),
                "data_mbr_target_reach_overview");
        return DataTargetHttpUtils.responseOfOne(res, null);
    }

    /**
     * 目标跟进核心指标趋势分月/累计
     * @param qo
     * @param chartType 1:分月 2：累计
     * @return
     */
    @RequestMapping(value = "get-target-chart")
    @ResponseBody
    public RestResponse getTargetChart(QueryDataStatisticsCommonQO qo, Integer chartType) {
        if (chartType == null) {
            chartType = 1;
        }
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("enterpriseId", qo.getEnterpriseId());
        Map<String, Object> res = DataTargetHttpUtils.http(DataTargetHttpUtils.getParamNoPage(jsonObject).toJSONString(),
                chartType == 1 ? "data_mbr_target_chart" : "data_mbr_target_cumulate_chart");
        return DataTargetHttpUtils.response(res);
    }

    /**
     * 目标跟进达成分析门店明细数据
     * @param qo
     * @param pageQo
     * @param sortField 排序字段
     * @param sortType asc desc
     * @param type 0:实际新增会员数 1:实际会员销售额 2：实际会员消费总次数 3：客单价 4：存量会员消费总次数 5：新增会员消费总次数 6：存量会员仅消费一次总次数 7：存量会员消费多次总次数
     *             8：新增会员仅消费一次总次数 9：新增会员消费多次总次数 10：存量会员仅消费一次总人数 11：存量会员消费多次总人数 12 ：量会员消费多次人均消费次数
     *             13 ：新增会员仅消费一次总人数 14：新增会员消费多次总人数 15：新增会员消费多次人均消费次数
     * @return
     */
    @RequestMapping(value = "get-target-analysis-store-detail")
    @ResponseBody
    public RestResponse getTargetAnalysisStoreDetail(QueryDataStatisticsCommonQO qo, PageQo pageQo,
                                                     String sortField, String sortType, @RequestParam(defaultValue = "1") Integer type) {
        JSONObject jsonObject = getCommon(qo, type <= 3 ? false : true);
        if (StringUtils.isNotBlank(sortField)) {
            jsonObject.put("orderByField", sortField + " " + sortType);
        }
        Map<String, Object> res = DataTargetHttpUtils.http(DataTargetHttpUtils.getParam(pageQo, jsonObject).toJSONString(),
                type <=3 ? "data_mbr_target_store_detail" :  (qo.getTimeType() == 1 ?
                        "data_mbr_target_analysis_year_store_detail" : "data_mbr_target_analysis_month_store_detail"));
        Page page = DataTargetHttpUtils.getPage(res);
        //加上总计数据
        RestResponse totalRes = getTargetAttainmentAnalysisOverview(qo);
        if (!"0000".equals(totalRes.getCode())) {
            return totalRes;
        }
        JSONObject totalJson = (JSONObject) totalRes.getResult();
        JSONObject total = new JSONObject();
        List<String> keyList = TargetAnalysisUtils.map.get(type.toString());
        for (String key : keyList) {
            total.put(key, totalJson.get(key));
        }
        total.put("storeId", "-1");
        total.put("storeName", "合计");
        total.put("storeCode", "合计");
        List<JSONObject> dataList = page.getResult();
        List<JSONObject> resultList = page.getResult();
        resultList.add(total);
        if (CollectionUtils.isNotEmpty(dataList)) {
            for (JSONObject obj : dataList) {
                JSONObject temp = new JSONObject();
                String storeId = obj.getString("storeId");
                temp.put("storeId", storeId);
                if (!"-1".equals(storeId)) {
                    StoreDTO storeDTO = storeService.getStore(storeId);
                    if (storeDTO != null) {
                        temp.put("storeName", storeDTO.getStoreName());
                        temp.put("storeCode", storeDTO.getStoreCode());
                    }
                }
                for (String key : keyList) {
                    temp.put(key, obj.get(key));
                }
                resultList.add(temp);
            }
        }
        page.setResult(resultList);
        return RestResponse.successResult(page);
    }

    /**
     * 目标跟进门店会员分层明细数据
     * @param qo
     * @return
     */
    @RequestMapping(value = "get-target-store-mbr-hierarchy-detail")
    @ResponseBody
    public RestResponse getTargetStoreMbrHierarchyDetail(QueryDataStatisticsCommonQO qo) {
        JSONObject jsonObject = getCommon(qo, true);
        Map<String, Object> res = DataTargetHttpUtils.http(DataTargetHttpUtils.getParamNoPage(jsonObject).toJSONString(),
                "data_mbr_target_store_mbr_hierarchy_detail");
        return DataTargetHttpUtils.responseOfOne(res, null);
    }

    /**
     * 目标跟进门店会员631分层明细数据
     * @param qo
     * @return
     */
    @RequestMapping(value = "get-target-store-mbr-631-detail")
    @ResponseBody
    public RestResponse getTargetStoreMbr631Detail(QueryDataStatisticsCommonQO qo) {
        JSONObject jsonObject = getCommon(qo, true);
        Map<String, Object> res = DataTargetHttpUtils.http(DataTargetHttpUtils.getParamNoPage(jsonObject).toJSONString(),
                "data_mbr_target_store_mbr_631_detail");
        return DataTargetHttpUtils.response(res);
    }

    private JSONObject getCommon(QueryDataStatisticsCommonQO qo, boolean app) {
        JSONObject jsonObject = new JSONObject();
        if (app) {
            jsonObject.put("entId", qo.getEnterpriseId());
            jsonObject.put("monthDate", qo.getTime());
        } else {
            jsonObject.put("enterpriseId", qo.getEnterpriseId());
            if (qo.getTimeType() == 1) {
                jsonObject.put("year", qo.getTime());
            } else {
                jsonObject.put("month", qo.getTime());
            }
        }
        if (StringUtils.isBlank(qo.getStoreId())) {
            List<String> storeIdList = storeAuthUtils.queryClerkStoreIds(qo.getClerkId(), qo.getWxEnterpriseId());
            jsonObject.put("storeId", storeIdList.stream().collect(Collectors.joining(",")));
        } else {
            jsonObject.put("storeId", qo.getStoreId());
        }
        return jsonObject;
    }
}
